#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Particle Systems" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

from math import pi, cos, sin
from random import random

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from vecteur import *


GRAVITY = 3.
NUM_PARTICLES = 1000
STEP_TIME = 0.01
PARTICLE_SIZE = 0.05


class Particule(object):
    def __init__(self):
        self.pos = 0
        self.velocity = 0
        self.color = 0
        

class ParticuleEngine(object):
    def __init__(self, textureId):
        self.textureId = textureId
        self.timeUntilNextStep = 0
        self.colorTime = 0
        self.angle = 0
        self.particules = []

        for i in range (NUM_PARTICLES) :
            particule = Particule()
            self.particules.append(particule)
            self.createParticule(particule)
        
        for i in range (int(5/STEP_TIME)) :
            self.step()


    def createParticule(self, p):
        p.pos = Vecteur()
        randomX = 0.5*random()-0.25
        randomY = 0.5*random()-0.25
        randomZ = 0.5*random()-0.25
        p.velocity = self.curVelocity() + Vecteur(randomX, randomY, randomZ)
        p.color = self.curColor()
        p.timeAlive = 0
        p.lifespan = random() + 1


    def curVelocity(self):
        return Vecteur(2*cos(self.angle), 2., 2*sin(self.angle))


    def curColor(self):
        if self.colorTime < 1/6. :
            color = Vecteur(1., self.colorTime*6, 0.)
        elif self.colorTime < 1/3. :
            color = Vecteur((1/3. - self.colorTime)*6, 1., 0.)
        elif self.colorTime < 1/2. :
            color = Vecteur(0., 1., (self.colorTime - 1/3.)*6)
        elif self.colorTime < 2/3. :
            color = Vecteur(0., (2/3. - self.colorTime)*6, 1.)
        elif self.colorTime < 5/6. :
            color = Vecteur((self.colorTime - 5/6.)*6, 0., 1.)
        else :
            color = Vecteur(1., 0., (1. - self.colorTime)*6)

        for coord in color.getCoord() :
            if coord < 0 : coord = 0
            elif coord > 1 : coord = 1
        
        return color


    def step(self):
        self.colorTime += STEP_TIME/10.

        while self.colorTime >= 1 : self.colorTime -= 1

        self.angle += 0.5*STEP_TIME
        while self.angle > 2*pi : self.angle -= 2*pi

        for p in self.particules :
            p.pos += p.velocity*STEP_TIME
            p.velocity += Vecteur(0., -GRAVITY*STEP_TIME, 0.)
            p.timeAlive += STEP_TIME

            if p.timeAlive > p.lifespan : self.createParticule(p)
           

    def draw(self):
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        
        glBegin(GL_QUADS)
        for p in self.particules :
            glColor4f(p.color._x, \
                      p.color._y, \
                      p.color._z, \
                      (1 - p.timeAlive/p.lifespan))
                
            size = PARTICLE_SIZE/2.
            pos = self.adjParticulePos(p.pos)
            
            x, y, z = pos.getCoord()
            glTexCoord2f(0, 0)
            glVertex3f(x - size, y - size, z)
            glTexCoord2f(0, 1)
            glVertex3f(x - size, y + size, z)
            glTexCoord2f(1, 1)
            glVertex3f(x + size, y + size, z)
            glTexCoord2f(1, 0)
            glVertex3f(x + size, y - size, z)
        glEnd()


    def adjParticulePos(self, pos):
        return self.rotate(pos, Vecteur(1, 0, 0), -30)

    
    def rotate(self, v, axis, degrees):
        radian = pi*degrees/180.
        axis = axis.unitaire()
        c = cos(radian)
        s = sin(radian)
        
        return v*c + axis*axis.scalaire(v)*(1.-c) + v.vectoriel(axis)*s
    

    def advance(self, dt):
        while dt > 0 :
            if self.timeUntilNextStep < dt :
                dt -= self.timeUntilNextStep
                self.step()
                self.timeUntilNextStep = STEP_TIME

            else :
                self.timeUntilNextStep -= dt
                dt = 0
