#!/usr/bin/python3
# -*-coding:utf-8-*-

from math import sqrt

class Vecteur(object):
    """
    Vecteur(x, y, z) -> vecteur à 3 dimensions.
    Nouveau vecteur par default :
        -> x = 0
        -> y = 0
        -> z = 0
    """

    def __init__(self, x=0, y=0, z=0):
        """
        Vecteur(x, y, z) -> vecteur à 3 dimensions.
        Initialisation du vecteur.
        """
        if not self._testInstance(x) or \
           not self._testInstance(y) or \
           not self._testInstance(z):
            raise ValueError("Erreur : valeurs numériques attendues.")

        self._x = x
        self._y = y
        self._z = z

 
    def getCoord(self):
        """
        v.getCoord() -> Renvoie les coordonnées du vecteur sous forme de list
        """
        return self._x, self._y, self._z

    
    def new(self, x, y, z):
        """
        V.new(x, y, z) -> nouveau vecteur avec x, y, z comme valeur.
        Renvoie erreur si x, y ou z n'est pas une valeur numérique.
        """
        if not self._testInstance(x) or \
           not self._testInstance(y) or \
           not self._testInstance(z):
            raise ValueError("Erreur : valeurs numériques attendues.")

        self._x = x
        self._y = y
        self._z = z


    def null(self):
        """
        V.null() -> vecteur nul avec x=0, y=0 and z=0.
        """
        self._x = 0
        self._y = 0
        self._z = 0


    def __add__(self, W):
        """
        V.__add__(W) -- renvoie le vecteur V + W.
        V et W vecteurs.
        Renvoie erreur si W n'est pas un vecteur.
        """
        if not isinstance(W, Vecteur):
            raise ValueError("Vecteur requis")

        A = Vecteur(self._x + W._x, \
                    self._y + W._y, \
                    self._z + W._z)

        return A


    def __mul__(self, n):
        """
        V.__mul__(n) -> renvoie le vecteur V * n.
        n valeur numérique.
        Renvoie erreur si n n'est pas une valeur numérique.
        """
        if not self._testInstance(n) :
            raise ValueError("Erreur : valeurs numériques attendues.")

        A = Vecteur(n * self._x, \
                    n * self._y, \
                    n * self._z)

        return A


    def __rmul__(self, n):
        """
        Pour la forme n * V.
        """
        return self * n


    def __imul__(self, n):
        """
        Pour la forme V *= n.
        """
        return self * n


    def __sub__(self, W):
        """
        V.__sub__(W) -> renvoie le vecteur V - W.
        V et W Vecteurs.
        Renvoie erreur si B n'est pas un vecteur.
        """
        if not isinstance(W, Vecteur):
            raise ValueError("Vecteur requis")

        A = Vecteur(self._x - W._x, \
                    self._y - W._y, \
                    self._z - W._z)

        return A


    def norme(self):
        """
        V.norme() -> renvoie la norme du vecteur.
        Norme = racine(x² + y² + z²)
        """
        return round(sqrt(self._x**2 + self._y**2 + self._z**2), 3)


    def unitaire(self):
        """
        V.unitaire() -> renvoie la vecteur unitaire.
                    |x = Vx/norme(V)
        Vunitaire = |y = Vy/norme(V)
                    |z = Vz/norme(V)
        """
        v_uni = Vecteur()
        norme = self.norme()

        if norme > 0:
            v_uni = Vecteur(self._x / norme, \
                            self._y / norme, \
                            self._z / norme)
        else:
            v_uni.null()

        return v_uni


    def scalaire(self, W):
        """
        V.scalaire(W) -> renvoie le produit scalaire.
        Scalaire = (V.x * W.x) + (V.y * W.y) + (V.z * W.z)
        Renvoie erreur si B n'est pas un vecteur.
        """
        if not isinstance(W, Vecteur):
            raise ValueError("Vecteur requis")

        return (self._x * W._x) + (self._y * W._y) + (self._z * W._z)


    def vectoriel(self, W):
        """
        V.vectoriel(W) -- renvoie le produit vectoriel sous forme de vecteur.
                            | (V.y * W.z) - (V.z * W.y)
        Produit vectoriel = | (V.z * W.x) - (V.x * W.z)
                            | (V.x * W.y) - (V.y * W.x)
        Renvoie erreur si W n'est pas un vecteur.
        """
        if not isinstance(W, Vecteur):
            raise ValueError("Vecteur requis")

        A = Vecteur((self._y * W._z) - (self._z * W._y), \
                    (self._z * W._x) - (self._x * W._z), \
                    (self._x * W._y) - (self._y * W._x))

        return A


    def __repr__(self, name=""):
        """
        Affiche le vecteur quand on entre directement le vecteur dans
        l'interpréteur.
        """
        l = len(name)
        return(" " * l + " | x = " + str(round(self._x, 3)) + "\n" \
               + name  + " | y = " + str(round(self._y, 3)) + "\n" + \
               " " * l + " | z = " + str(round(self._z, 3)) + "\n")


    def __str__(self):
        """
        Affiche le vecteur lors du print.
        """
        return self.__repr__()


    def affich(self, name='V'):
        """
        Affiche le vecteur avec son nom devant.
        """
        print(self.__repr__(name))
        

    def _testInstance(self, n):
        """
        Test les valeurs reçues si integer or float.
        """
        if isinstance(n, float) or isinstance(n, int):
            return True
        else:
            return False


if __name__ == "__main__":
    A = Vecteur()
    B = Vecteur()

    #Création du vecteur.
    A.new(1, 1, 1)
    B.new(1, 2, 3)
